library IEEE;
use IEEE.std_logic_1164.all;

entity Control_Exhibidores_7s is
   port(
      RST : in  std_logic;                    -- Reset maestro
      CLK : in  std_logic;                    -- Reloj maestro
      H3  : in  std_logic_vector(3 downto 0); -- Entrada 3
      H2  : in  std_logic_vector(3 downto 0); -- Entrada 2
      H1  : in  std_logic_vector(3 downto 0); -- Entrada 1 
      H0  : in  std_logic_vector(3 downto 0); -- Entrada 0
      P   : in  std_logic_vector(3 downto 0); -- Entrada puntos
      A   : out std_logic_vector(3 downto 0); -- Anodos
      S   : out std_logic_vector(7 downto 1); -- 7 segmentos
      PD  : out std_logic                     -- Punto decimal
      );
   end Control_Exhibidores_7s;

architecture Completa of Control_Exhibidores_7s is

-- Base de tiempo de 5 kHz
component Modulo_10000
   port(
      RST : in  std_logic; -- Reset maestro
      CLK : in  std_logic; -- Reloj maestro
      Q   : out std_logic  -- Cero
      );
   end component;

-- Contador modulo 4
component Modulo_4
   port(
      RST : in  std_logic;                    -- Reset maestro
      CLK : in  std_logic;                    -- Reloj maestro
      H   : in  std_logic;                    -- Habilitacion de entrada
      Q   : out std_logic_vector(1 downto 0)  -- Salida
      );
   end component;

-- Demultiplexor 2 a 4
component Demux_2_4
   port(
      Q : in  std_logic_vector(1 downto 0); -- Anodo activo
      Y : out std_logic_vector(3 downto 0)  -- Anodos
      );
   end component;

-- Multiplexor cuadruple de 4 a 1
component Mux4_4_1
   port(
      Q  : in  std_logic_vector(1 downto 0); -- Selector
      H3 : in  std_logic_vector(3 downto 0); -- Entrada 3
      H2 : in  std_logic_vector(3 downto 0); -- Entrada 2
      H1 : in  std_logic_vector(3 downto 0); -- Entrada 1
      H0 : in  std_logic_vector(3 downto 0); -- Entrada 0
      Y  : out std_logic_vector(3 downto 0)  -- Salida
      );
   end component;

-- Multiplexor de 4 a 1
component Mux_4_1
   port(
      Q  : in  std_logic_vector(1 downto 0); -- Selector
      P3 : in  std_logic;                    -- Entrada 3
      P2 : in  std_logic;                    -- Entrada 2
      P1 : in  std_logic;                    -- Entrada 1
      P0 : in  std_logic;                    -- Entrada 0
      PD : out std_logic                     -- Salida
      );
   end component;

-- Decodificador de hexadecimal a 7 segmentos
component Hexadecimal_7s
   port(
      H : in  std_logic_vector(3 downto 0); -- Hexadecimal
      S : out std_logic_vector(7 downto 1)  -- 7 segmentos
      );
   end component;

-- Habilitacion interna
signal ENI : std_logic;

-- Contador modulo 4
signal Q   : std_logic_vector(1 downto 0);

-- Multiplexor de los segmentos
signal M   : std_logic_vector(3 downto 0);

begin

   -- Base de tiempo
   Bloque_01: Modulo_10000   port map(RST,CLK,ENI);

   -- Contador modulo 4
   Bloque_02: Modulo_4       port map(RST,CLK,ENI,Q);

   -- Demultiplexor 2 a 4
   Bloque_03: Demux_2_4      port map(Q,A);

   -- Multiplexor de los segmentos
   Bloque_04: Mux4_4_1       port map(Q,H3,H2,H1,H0,M);

   -- Multiplexor del punto decimal
   Bloque_05: Mux_4_1        port map(Q,P(3),P(2),P(1),P(0),PD);

   -- Decodificador de hexadecimal a 7 segmentos
   Bloque_06: Hexadecimal_7s port map(M,S);

end Completa;
