library IEEE;
use IEEE.std_logic_1164.all;

entity Interfaz is
   port(
      RST : in    std_logic;                     -- Reset maestro
      CLK : in    std_logic;                     -- Reloj maestro
      VAD : in    std_logic_vector(8  downto 0); -- Direccion vertical
      HAD : in    std_logic_vector(9  downto 0); -- Direccion horizontal
      PIX : out   std_logic_vector(2  downto 0); -- Pixel activo
      STG : out   std_logic_vector(1  downto 0); -- Control del generador
      EOG : in    std_logic;                     -- Fin de generacion
      BAR : in    std_logic_vector(17 downto 0); -- Direccion de la barra
      KLR : in    std_logic_vector(5  downto 0); -- Colores de la barra
      D   : inout std_logic_vector(7  downto 0); -- Datos RAM
      A   : out   std_logic_vector(17 downto 0); -- Direcciones RAM
      CE1 : out   std_logic;                     -- Chip enable 1
      CE2 : out   std_logic;                     -- Chip enable 2
      UB1 : out   std_logic;                     -- Upper byte enable 1
      UB2 : out   std_logic;                     -- Upper byte enable 2
      LB1 : out   std_logic;                     -- Lower byte enable 1
      LB2 : out   std_logic;                     -- Lower byte enable 2
      WR  : out   std_logic;                     -- Write enable
      OE  : out   std_logic                      -- Output enable
      );
   end Interfaz;

architecture Control of Interfaz is

-- Estados internos
signal Qp, Qn : std_logic_vector(2  downto 0);

-- Mux de direcciones
signal S      : std_logic;

begin

   FSM: process(Qp,EOG)
   begin
      CE2 <= '1';
      UB2 <= '1';
      UB1 <= '1';
      LB2 <= '1';
      LB1 <= '0';
      case Qp is
         when "000"  =>
            CE1 <= '1';
            WR  <= '1';
            OE  <= '1';
            S   <= '0';
            STG <= "11";
            Qn  <= "001";
         when "001"  =>
            CE1 <= '0';
            WR  <= '1';
            OE  <= '1';
            S   <= '0';
            STG <= "00";
            Qn  <= "010";
         when "010"  =>
            CE1 <= '0';
            WR  <= '0';
            OE  <= '1';
            S   <= '0';
            STG <= "00";
            Qn  <= "011";
         when "011"  =>
            CE1 <= '0';
            WR  <= '1';
            OE  <= '1';
            S   <= '0';
            STG <= "00";
            if (EOG='0') then
               Qn  <= "100";
            else
               Qn  <= "101";
            end if;
         when "100"  =>
            CE1 <= '0';
            WR  <= '1';
            OE  <= '1';
            S   <= '0';
            STG <= "10";
            Qn  <= "001";
         when "101"  =>
            CE1 <= '0';
            WR  <= '1';
            OE  <= '1';
            S   <= '1';
            STG <= "11";
            Qn  <= "110";
         when "110"  =>
            CE1 <= '0';
            WR  <= '1';
            OE  <= '0';
            S   <= '1';
            STG <= "11";
            Qn  <= "110";
         when others =>
            CE1 <= '1';
            WR  <= '1';
            OE  <= '1';
            S   <= '0';
            STG <= "11";
            Qn  <= "000";
      end case;
   end process FSM;

   Direccion: process(VAD,HAD,BAR,S)
   begin
      if (S='0') then
         A <= BAR;
      else
         A <= VAD & HAD(9 downto 1);
      end if;
   end process Direccion;

   Dato: process(D,KLR,S,HAD)
   begin
      if (S='0') then
         D <= "00" & KLR;
      else
         D <= (others => 'Z');
      end if;
      if (HAD(0)='0') then
         PIX <= D(5 downto 3);
      else
         PIX <= D(2 downto 0);
      end if;
   end process Dato;

   Reloj: process(RST,CLK)
   begin
      if (RST='1') then
         Qp <= (others => '0');
      elsif (CLK'event and CLK='1') then
         Qp <= Qn;
      end if;
   end process Reloj;

end Control;
