library IEEE;
use IEEE.std_logic_1164.all;

entity Ejemplo_VGA_2 is
   port(
      RST : in    std_logic;                     -- Reset maestro
      CLK : in    std_logic;                     -- Reloj maestro
      D   : inout std_logic_vector(7  downto 0); -- Datos RAM
      A   : out   std_logic_vector(17 downto 0); -- Direcciones RAM
      CE1 : out   std_logic;                     -- Chip enable 1
      CE2 : out   std_logic;                     -- Chip enable 2
      UB1 : out   std_logic;                     -- Upper byte enable 1
      UB2 : out   std_logic;                     -- Upper byte enable 2
      LB1 : out   std_logic;                     -- Lower byte enable 1
      LB2 : out   std_logic;                     -- Lower byte enable 2
      WR  : out   std_logic;                     -- Write enable
      OE  : out   std_logic;                     -- Output enable
      VS  : out   std_logic;                     -- Sincronia vertical
      HS  : out   std_logic;                     -- Sincronia horizontal
      R   : out   std_logic;                     -- Red
      G   : out   std_logic;                     -- Green
      B   : out   std_logic                      -- Blue
      );
   end Ejemplo_VGA_2;

architecture Prueba of Ejemplo_VGA_2 is

-- Sincronizador VGA
component Sincronizador_VGA
   port(
      RST : in  std_logic;                    -- Reset maestro
      CLK : in  std_logic;                    -- Reloj maestro
      VS  : out std_logic;                    -- Sincronia vertical
      HS  : out std_logic;                    -- Sincronia horizontal
      VAD : out std_logic_vector(8 downto 0); -- Direccion vertical
      HAD : out std_logic_vector(9 downto 0); -- Direccion horizontal
      EDS : out std_logic                     -- Habilitacion pixel
      );
   end component;

-- Habilitacion de pixel
component Control_Pixel
   port(
      EDS : in  std_logic;                    -- Habilitacion
      PIX : in  std_logic_vector(2 downto 0); -- Color
      R   : out std_logic;                    -- Red
      G   : out std_logic;                    -- Green
      B   : out std_logic                     -- Blue
      );
   end component;

-- Generador de barras
component Genera_Barras
   port(
      RST : in  std_logic;                     -- Reset maestro
      CLK : in  std_logic;                     -- Reloj maestro
      STG : in  std_logic_vector(1  downto 0); -- Control del generador
      EOG : out std_logic;                     -- Fin de generacion
      BAR : out std_logic_vector(17 downto 0); -- Direccion de la barra
      KLR : out std_logic_vector(5  downto 0)  -- Colores de la barra
      );
   end component;

-- Interfaz
component Interfaz
   port(
      RST : in    std_logic;                     -- Reset maestro
      CLK : in    std_logic;                     -- Reloj maestro
      VAD : in    std_logic_vector(8  downto 0); -- Direccion vertical
      HAD : in    std_logic_vector(9  downto 0); -- Direccion horizontal
      PIX : out   std_logic_vector(2  downto 0); -- Pixel activo
      STG : out   std_logic_vector(1  downto 0); -- Control del generador
      EOG : in    std_logic;                     -- Fin de generacion
      BAR : in    std_logic_vector(17 downto 0); -- Direccion de la barra
      KLR : in    std_logic_vector(5  downto 0); -- Colores de la barra
      D   : inout std_logic_vector(7  downto 0); -- Datos RAM
      A   : out   std_logic_vector(17 downto 0); -- Direcciones RAM
      CE1 : out   std_logic;                     -- Chip enable 1
      CE2 : out   std_logic;                     -- Chip enable 2
      UB1 : out   std_logic;                     -- Upper byte enable 1
      UB2 : out   std_logic;                     -- Upper byte enable 2
      LB1 : out   std_logic;                     -- Lower byte enable 1
      LB2 : out   std_logic;                     -- Lower byte enable 2
      WR  : out   std_logic;                     -- Write enable
      OE  : out   std_logic                      -- Output enable
      );
   end component;

-- Sincronizador VGA
signal VAD: std_logic_vector(8 downto 0);
signal HAD: std_logic_vector(9 downto 0);
signal EDS: std_logic;

-- Habilitacion de pixel
signal PIX: std_logic_vector(2 downto 0);

-- Generador de barras
signal STG: std_logic_vector(1  downto 0);
signal EOG: std_logic;
signal BAR: std_logic_vector(17 downto 0);
signal KLR: std_logic_vector(5  downto 0);


begin

   -- Sincronizador VGA
   DUT_01: Sincronizador_VGA port map(RST,CLK,VS,HS,VAD,HAD,EDS);

   -- Habilitacion de pixel
   DUT_02: Control_Pixel     port map(EDS,PIX,R,G,B);

   -- Generador de barras
   DUT_03: Genera_Barras     port map(RST,CLK,STG,EOG,BAR,KLR);

   -- Interfaz
   DUT_04: Interfaz          port map(RST,CLK,VAD,HAD,PIX,STG,EOG,BAR,KLR,D,A,CE1,CE2,UB1,UB2,LB1,LB2,WR,OE);

end Prueba;